"""
Custom demographic model for our simulation.
"""
import numpy
import dadi

def prior_twopop_bottleneck_mig((nu1B, nu1F, nu2B, nu2F, nuA, m, Tp, T), (n1,n2), pts):
    """
    Model with growth, split, bottleneck in pop2 , exp recovery, migration
    nuA : The ancestral population size after growth. (Initial size is 1.)
    nu1B: The bottleneck size for pop1
    nu1F: The final size for pop1 (The ancestral population size after growth is nu1F)
    nu2B: The bottleneck size for pop2
    nu2F: The final size for pop2
    m: The scaled migration rate
    Tp: The scaled time between ancestral population growth and the split.
    T: The time between the split and present
    n1,n2: Size of fs to generate.
    pts: Number of points to use in grid for evaluation.
    """
    # Define the grid we'll use
    xx = yy = dadi.Numerics.default_grid(pts)

    # phi for the equilibrium ancestral population
    phi = dadi.PhiManip.phi_1D(xx)
    # Now do the population growth event.
    phi = dadi.Integration.one_pop(phi, xx, Tp, nu=nuA)

    # The divergence
    phi = dadi.PhiManip.phi_1D_to_2D(xx, phi)
    # We need to define a function to describe the non-constant population
    # size. lambda is a convenient way to do so.
    nu1_func = lambda t: nu1B*(nu1F/nu1B)**(t/T)
    nu2_func = lambda t: nu2B*(nu2F/nu2B)**(t/T)
    phi = dadi.Integration.two_pops(phi, xx, T, nu1=nu1_func, nu2=nu2_func, 
                                    m12=m, m21=m)

    # Finally, calculate the spectrum.
    sfs = dadi.Spectrum.from_phi(phi, (n1,n2), (xx,yy))
    return sfs

def prior_bottleneck_nonbottleneck_mig((nu1S, nu1B, nu1F, nu2F, nuA, m, Tp, T1, T2),(n1,n2),pts):
    """
    Model with growth, split, bottleneck in pop2 , exp recovery, migration
    nuA : The ancestral population size after growth. (Initial size is 1.)
    nu1B: The bottleneck size for pop1
    nu1F: The final size for pop1
    nu1S: The size for pop1 after split
    nu2F: The final size for pop2 (The ancestral population size after growth is nu2F)
    m: The scaled migration rate
    Tp: The scaled time between ancestral population growth and the split.
    T1: The time between the split and bottleneck
    T2: The time between bottleneck and present. (Tsplit = T1+T2)
    n1,n2: Size of fs to generate.
    pts: Number of points to use in grid for evaluation.
    """
    xx = yy = dadi.Numerics.default_grid(pts)

    # phi for the equilibrium ancestral population
    phi = dadi.PhiManip.phi_1D(xx)
    # Now do the population growth event.
    phi = dadi.Integration.one_pop(phi, xx, Tp, nu=nu2F)

    # The divergence
    phi = dadi.PhiManip.phi_1D_to_2D(xx, phi)
    # We need to define a function to describe the non-constant population
    # size. lambda is a convenient way to do so.
    def nu1_func((nu1S,nu1B,nu1F, T1, T2),t):
      if t<T1:
        return nu1S*(nu1B/nu1S)**(t/T1)
      if t>=T1 and t<(T1+T2):
        return nu1B*(nu1F/nu1B)**((t-T1)/T2)
    phi = dadi.Integration.two_pops(phi, xx, T1+T2, nu1=nu1_func, nu2=nu2F,
                                    m12=m, m21=m)

    # Finally, calculate the spectrum.
    sfs = dadi.Spectrum.from_phi(phi, (n1,n2), (xx,yy))
    return sfs

